// Include appropriate libraries
#include <iostream>
#include <vector>

#define _USE_MATH_DEFINES
#include <math.h>

#include <GL/glut.h>
#include <ctime>

// Include other classes required
#include "Border.h"
#include "LightCycle.h"
#include "Trail.h"
#include "Disc.h"
#include "graphicsUtils.h"

// Grab access to the global list of pointers to objects
extern vector< LightCycle* > gCycles;
extern vector< Trail* > gTrails;
extern vector< Disc* > gDiscs;
extern vector< Border* > gBorders;

// Border CLASS 

Border::Border( float x1 = 0.0, float x2 = 0.0, float y1 = 0.0, float y2 = 0.0 )
{
	left = x1;
	right = x2;
	top = y1;
	bottom = y2;
	
	hit = 0;
}

// Draw border
void Border::drawBorder()
{
	glPushMatrix();
		glBegin( GL_QUADS );
			if (hit == 0)
			{
				glColor3f( 0.0, 0.0, 0.0 );
			}
			if (hit == 1)
			{
				glColor3f( 1.0, 1.0, 1.0 );
			}
			glVertex3f( left, top, 0.0 );	
			glVertex3f( left, bottom, 0.0 );	
			glVertex3f( right, bottom, 0.0 );	
			glVertex3f( right, top, 0.0 );	
		glEnd();
	glPopMatrix();
}

// Compute intersection of light cycles and discs with border
void Border::computeIntersection( int On )
{	
	hit = 0;
	
	float templeft, tempright, temptop, tempbottom;
	
	// Tanks
	for( unsigned int i = 0; i < gCycles.size(); i++)
	{
		templeft = left - gCycles[i]->sizeL;
		tempright = right + gCycles[i]->sizeL;
		temptop = top + gCycles[i]->sizeL;
		tempbottom = bottom - gCycles[i]->sizeL;
		
		// If tank hits barrier, set it back to last position
		if (gCycles[i]->position[0] > templeft && gCycles[i]->position[0] < tempright && gCycles[i]->position[1] < temptop && gCycles[i]->position[1] > tempbottom)
		{
			gCycles[i]->position[0] = gCycles[i]->prevPosition[0];
			gCycles[i]->position[1] = gCycles[i]->prevPosition[1];
			
			hit = 1;
			gCycles[i]->lives -= 1;

		}
	}
	
	if (On == 0)
	{
		// Discs
		float templeft, tempright, temptop, tempbottom;
		
		for( unsigned int k = 0; k < gDiscs.size(); k++)
		{
			templeft = left - gDiscs[k]->size;
			tempright = right + gDiscs[k]->size;
			temptop = top + gDiscs[k]->size;
			tempbottom = bottom - gDiscs[k]->size;
			
			// If disc hits border, it bounces
			if (gDiscs[k]->position[0] > templeft && gDiscs[k]->position[0] < tempright && gDiscs[k]->position[1] < temptop && gDiscs[k]->position[1] > tempbottom)
			{
				
				if (gDiscs[k]->prevPosition[0] < templeft || gDiscs[k]->prevPosition[0] > tempright)
				{
					gDiscs[k]->direction[0] *= -1;
					
				}
				if (gDiscs[k]->prevPosition[1] > temptop || gDiscs[k]->prevPosition[1] < tempbottom)
				{
					gDiscs[k]->direction[1] *= -1;
				}
			}
		}
	}
}

