// Include appropriate libraries
#include <iostream>
#include <vector>
using namespace std;

#include <GL/glut.h>

#include <math.h>
#include <ctime>
#include <iostream>

// Include other classes required
#include "LightCycle.h"
#include "Trail.h"
#include "graphicsUtils.h"
#include "Disc.h"

using namespace std;

// Grab access to the global list of pointers to objects
extern vector< LightCycle* > gCycles;
extern vector< Trail* > gTrails;
extern vector< Disc* > gDiscs;

Disc::Disc( float x = 0.0, float y = 0.0, float z = 0.0, float color0 = 0.0, float color1 = 0.0, float color2 = 0.0 )
{
	// declare data members

	speed = 0.07;
	size = 0.2;
	angle = 0;
	maxRange = 2.0;
	
	spent = 0;
	
	direction[0] = 1.0;
	direction[1] = -1.0;
	direction[2] = 1.0;
	
	totalTraveled = 0.0;
	
	position[0] = x;
	position[1] = y;
	position[2] = z;
	
	defaultPosition[0] = 0.0;
	defaultPosition[1] = 0.0;
	defaultPosition[2] = 0.0;

	prevPosition[0] = 0.0;
	prevPosition[1] = 0.0;
	prevPosition[2] = 0.0;

	color[0] = color0;
	color[1] = color1;
	color[2] = color2;

	elapsedTime = 1.0;

}

// Compute position of disc
// If the shell has reached max distance of travel or has hit a cycle, delete it
// If not, set position and direction
void Disc::computePosition( double elapsed_time, int iPos )
{
	float distanceTraveled;
	float distanceTraveledVector[3];

	if ( elapsed_time > MINIMUM_MILLISECOND_UPDATE )
	{
		// keep track of total distance traveled
		distanceTraveled = speed * elapsedTime;
		totalTraveled += distanceTraveled;

		if ( totalTraveled > maxRange || spent )
		{
			gDiscs.erase(gDiscs.begin() + iPos);
			delete(this);
		}
		else
		{	
			distanceTraveledVector[0] = distanceTraveled * direction[0] * sin(angle * M_PI / 180);
			distanceTraveledVector[1] = distanceTraveled * direction[1] * cos(angle * M_PI / 180);
			distanceTraveledVector[2] = distanceTraveled * direction[2];

			prevPosition[0] = position[0];
			prevPosition[1] = position[1];
			prevPosition[2] = position[2];			
			
			position[0] += distanceTraveledVector[0];
			position[1] += distanceTraveledVector[1];
			position[2] += position[2];
		}
		
	}

}

// Draw the shell
void Disc::drawDisc()
{
	glPushMatrix();
		
		glTranslatef( position[0], position[1], position[2] );
		
		glColor3f( color[0], color[1], color[2] );
		glutSolidSphere( size / 2, 20, 20 );
		
		glTranslatef( 0.0, 0.0, 0.0 + 0.1 );
		
		glColor3f( 0.0, 0.0, 0.0 );
		glutSolidSphere( size / 2.5, 20, 20 );
		
		glTranslatef( 0.0, 0.0, 0.0 + 0.11 );
		
		glColor3f( color[0], color[1], color[2] );
		glutSolidSphere( size / 4.5, 20, 20 );
		
		glTranslatef( 0.0, 0.0, 0.0 + 0.12 );
		
		glColor3f( 0.0, 0.0, 0.0 );
		glutSolidSphere( size / 5, 20, 20 );
		
	glPopMatrix();
}

// Determine if disc has hit tank
// If it has delete the shell and end the game
void Disc::computeHitCycle( int iPos )
{
	float templeft, tempright, temptop, tempbottom;
	
	for ( unsigned int i = 0; i < gCycles.size(); ++i )
	{
		templeft = position[0] - (size / 2) - gCycles[i]->sizeW;
		tempright = position[0] + (size / 2) + gCycles[i]->sizeW;
		temptop = position[1] + (size / 2) + gCycles[i]->sizeL;
		tempbottom = position[1] - (size / 2) - gCycles[i]->sizeL;
		
		// If disc hits light cycle, it bounces off and the user hit loses the game
		if (gCycles[i]->position[0] > templeft && gCycles[i]->position[0] < tempright && gCycles[i]->position[1] < temptop && gCycles[i]->position[1] > tempbottom)
		{
			
			if (gCycles[i]->prevPosition[0] < templeft || gCycles[i]->prevPosition[0] > tempright)
			{
				gDiscs[iPos]->direction[0] *= -1;	
			}
			if (gCycles[i]->prevPosition[1] > temptop || gCycles[i]->prevPosition[1] < tempbottom)
			{
				gDiscs[iPos]->direction[1] *= -1;
			}
			
			gCycles[i]->lives -= 1;
		}
	}
}
